# frozen_string_literal: true

require "is/inspectable"

require_relative "../../is/async"

module Core
  module Async
    class Enumerator
      include Enumerable
      include Is::Async
      include Is::Inspectable

      def initialize(object)
        unless object.respond_to?(:each)
          raise ArgumentError, "object is not enumerable"
        end

        @object = object
      end

      # [public] Yields each value within its own async context, waiting on the enumeration to complete.
      #
      def each
        unless block_given?
          return to_enum(:each)
        end

        await do
          errored, stopped = false

          @object.each do |value|
            break if errored || stopped

            async do
              yield value
            rescue LocalJumpError
              stopped = true
            rescue => error
              errored = true
              raise error
            end
          end
        end
      end
    end
  end
end
