# frozen_string_literal: true

require "is/inspectable"

require_relative "../../is/async"

module Core
  module Async
    # [public] The top-level async context. Runs until all scheduled work is complete.
    #
    class Reactor
      include Is::Async
      include Is::Inspectable
      inspects without: [:@runnable]

      class << self
        # [public] Create a new reactor and immediately run it.
        #
        def run(&block)
          instance = new
          instance.run(&block)
        end
      end

      # [public] Run the reactor, yielding within the async context.
      #
      def run
        if (task = ::Async::Task.current?)
          reference = task.async { |child|
            @runnable = child

            yield self
          }

          wait_all(reference)
        else
          @runnable = ::Async::Reactor.new

          @runnable.run {
            async {
              yield self
            }.result
          }.wait
        end
      end

      # [public] Stop the reactor.
      #
      def stop
        @runnable&.stop
      end
    end
  end
end
