# frozen_string_literal: true

module Core
  module Async
    # [public] Wraps an object, calling all methods in an async context.
    #
    class Wrapper < BasicObject
      def initialize(target)
        unless target.respond_to?(:async)
          ::Kernel.raise ::ArgumentError, "object is not async-aware"
        end

        @target = target
      end

      def method_missing(name, *args, &block)
        @target.async do
          @target.public_send(name, *args, &block)
        end
      end

      ruby2_keywords(:method_missing) if respond_to?(:ruby2_keywords)

      def respond_to_missing?(name, *)
        true
      end
    end
  end
end
